const { plugin: { pluginObjectTypes, pluginScope } } = require('@kl/constants')
const lo = require('lodash')
const path = require('path')
const fs = require('fs')
const consts = require('./consts')
const xml2js = require('xml2js')

function PluginServerAPI (kscApi) {
  const policy = require('./converters/policy/index')()
  const settings = require('./converters/settings/index')()
  const tasksGroup = require('./converters/tasks/index.js')()
  const tasksLocal = require('./converters/local_tasks/index.js')()
  const distributive = require('./converters/distributive/index')()

  async function getFormModel ({ context: { object: { id, type, parentId }, scope }, form, type: taskType }) {
        console.log('kess_windows:> getFormModel ' + JSON.stringify(arguments))
        var data = { }
        switch (type) {
            case pluginObjectTypes.policy:
                data = await policy.getPolicyData(kscApi, id, parentId, form);
                break
            case pluginObjectTypes.task:
                data = await tasksGroup.getTaskData ({kscApi, taskId: id, hostId: false, taskType, formType: form})
                break
            case pluginObjectTypes.taskInitial:
                let defData = await getTaskDefaults({taskType, scope})
                data = await tasksGroup.getWizardTaskData({kscApi, taskType, formType: form, defData})
                break
            case pluginObjectTypes.localTask:
                data = await tasksLocal.getTaskData(kscApi, id, parentId, taskType, form, type);
                break
            case pluginObjectTypes.settings:
                data = await settings.getSettingsData(kscApi, form, parentId, type)
                break
            case pluginObjectTypes.distributive:
                data = await distributive.getPackageData ({kscApi, packageId: id, form})
                break
        }
        return data
  }

  async function setFormModel ({ context: { scope, object: { type, parentId } }, form, taskType }, data) {
      console.log('kess_windows:> setFormModel ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              await policy.updatePolicyData(kscApi, form, data)
              break
          case pluginObjectTypes.task:
              if (scope === pluginScope.editObject)
                  await tasksGroup.updateTaskData ({kscApi, modelData: data, taskType, formType: form})
              break
          case pluginObjectTypes.taskInitial:
              await tasksGroup.updateTaskData ({kscApi, modelData: data, taskType, formType: form})
              break
          case pluginObjectTypes.localTask:
              if (scope === pluginScope.editObject)
                  await tasksLocal.updateTaskData ({kscApi, modelData: data, taskType, formType: form, hostId: parentId})
              break
          case pluginObjectTypes.settings:
              await settings.updateSettingsData(kscApi, form, data)
              break
          case pluginObjectTypes.distributive:
              data = await distributive.savePackageData ({kscApi, modelData: data, form})
              break
      }
      return true
  }

  async function beforeApplyChanges ({ object: { id, type }, scope }) {
      console.log('kess_windows:> beforeApplyChanges ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              policy.commitPolicyData(kscApi, scope)
              break
          case pluginObjectTypes.settings:
              settings.commitSettingsData(kscApi)
              break
      }
  }

  async function applyChanges ({ object: { id, type }, scope }) {
      console.log('kess_windows:> applyChanges ' + JSON.stringify(arguments))
      switch (type) {
          case pluginObjectTypes.policy:
              await policy.savePolicyData(kscApi, id, scope)
              break
          case pluginObjectTypes.settings:
              await settings.saveSettingsData(kscApi, id)
              break
      }
  }

  /**
   * Метод для получения данных от плагина по имени файла
   * @param fileName - имя файла с запрашиваемыми данными
   * @returns {Promise<string | Buffer>} - текстовое содержимое файла
   */
  async function getAssetByFileName ({fileName}) {
    try {
      const parsedFileName = path.parse(fileName)
      const filePath = path.join(__dirname, 'assets', parsedFileName.base)
      return fs.readFileSync(filePath, 'utf-8')
    } catch (error) {
      console.error(error)
    }
  }

  /**
   * Запуск продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function startProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Остановка продукта на устройстве
   * @param {string} hostName - уникальное имя хоста
   * @return {Promise.<*>}
   */
  async function stopProductOnHost ({ hostName }) {
    return hostName
  }

  /**
   * Возвращает параметры задачи по-умолчанию
   * @param taskType - тип задачи
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskDefaults({ taskType, scope }) {
      console.log('kess_windows:> getTaskDefaults ' + JSON.stringify(arguments))
      switch (scope) {
          case pluginScope.newObject:
          case pluginScope.newQSWObject:
          case pluginScope.newCloudProtectionObject:
              const tasksRaw = await getAssetByFileName({ fileName: 'task-defaults.json' });
              const tasks = JSON.parse(tasksRaw);
              return tasks[taskType];
      }
      return {}
  }

  /**
   * Возвращает продуктовые настройки задачи
   * @param taskData - настройки задачи KSC
   * @param scope - контекст выполнения операции
   * @returns {Promise.<*>}
   */
  async function getTaskAdditionalParams ({ task, scope }) {
    console.log('kess_windows:> getTaskAdditionalParams ' + JSON.stringify(arguments))
      switch (scope) {
          case pluginScope.newObject:
              return tasksGroup.getTaskAdditionalParams()
              break
          case pluginScope.editObject:
              if(task.isLocalTask)
                  return tasksLocal.getTaskAdditionalParams(task.TASK_NAME)
              else
                  return tasksGroup.getTaskAdditionalParams()
              break
          case pluginScope.newQSWObject:
              return task.TASK_ADDITIONAL_PARAMS
              break

      }
   }

  /**
   * Return task registration data
   * @param taskType - required task type
   * @returns {Promise.<object>}
   */
  async function getTaskRegistrationData ({ taskType }) {
    const taskTypesJson = await getAssetByFileName({ fileName: 'task-types.json' })
    const taskTypes = JSON.parse(taskTypesJson)
    const taskData = lo.find(taskTypes, { type: taskType })
    if (!taskData) {
      throw new Error('Unknown task type')
    }
    return taskData
  }

  // internal methods
  function parseXMLtoJSON({ xml }) {
     return new Promise((resolve, reject) => {
        xml2js.parseString(xml, function (err, result) {
           err ? reject(err) : resolve(result)
        });
     })
  }

  async function callApi ({ command, params }) {
     const [
        openApiInterface,
        openApiMethod
     ] = command.split('.')
     const response = await kscApi[openApiInterface][openApiMethod](params)
     return response.result
  }

   async function EnumLicenseKeys() {
        const licFields = ['KLLIC_APP_ID','KLLIC_KEY_TYPE','KLLIC_SERIAL','KLLICSRV_KEY_INSTALLED','KLLICSRV_KEYDATA'];
        const response = await kscApi.LicenseKeys.EnumKeys({ pFields: licFields })
        return response.result
   }

   async function GetKeyData({serial}) {
        const response = await kscApi.LicenseKeys.GetKeyData({
            pKeyInfo: {'KLLIC_SERIAL': serial, 'KLLICSRV_KEYDATA': true}
        })
        return response.result
   }

    async function IsKpsnMode() {
        const eulas = await kscApi.KsnInternal.GetNKsnEulas()
        return (eulas.result!==undefined && eulas.result.pNKsnEulas!==undefined && eulas.result.pNKsnEulas.length > 0 ) ? true : false
    }

    return {
    getFormModel,
    setFormModel,
    beforeApplyChanges,
    applyChanges,
    getAssetByFileName,
    startProductOnHost,
    stopProductOnHost,
    getTaskAdditionalParams,
    getTaskDefaults,
    getTaskRegistrationData,
    internal: {
        parseXMLtoJSON,
        EnumLicenseKeys,
        GetKeyData,
        IsKpsnMode,
        callApi
    }
  }
}

module.exports = PluginServerAPI